
%% Inputs:
%% r = 'pi' in the paper; c = tuning parameter of psi function,  
%% df = distribution of the contamination: t1 (Cauchy), 
%% t3 (t on 3 degrees of freedon) or N9 (Normal with a variance of 9)     
%% k = tuning parameter in the weights, 
%% Regtype = linear ('L'), quadratic ('Q') or cubic ('C');   
%% Mtype = Mallows ('M') or Schweppe ('S'); 
%% psitype = Huber ('H') or bisquare ('B');  
%% VARY = 1 if r is varied,  = 2 if c is varied, 
%% = 3 if k is varied, = 4 if eps is varied, = 5 if df is varied.
%% When varying df set, e.g df = [1,2,3] to mean ['t1', 't3', 'N9']
%% The heading Figure 1, Figure 2, etc. refer to the 
%% figure numbers in the paper

%% Run all of these to get the final plots:

%% Figure 1:
%% [Frequencies1, Measures1] = Mdesign(4000, [0, .05, .5, .85], 20, 20, 5000, 0, 't1', 0, 'L','M', 'H', 1)

%% Figure 2:
%% [Frequencies2a, Measures2a] = Mdesign(4000, .5, 20, 20, [.5, 1.5, 5], .25, 't1', 10, 'L','M', 'H', 2)
%% [Frequencies2b, Measures2b] = Mdesign(4000, .5, 20, 20, [.5, 1.5, 5], .25, 't1', 10, 'L','S', 'H', 2)

%% Figure 3:
%% [Frequencies3a, Measures3a] = Mdesign(4000, .05, 20, 20, 1.5, .25, 't1', [0,10,20], 'L', 'M', 'H', 3)
%% [Frequencies3b, Measures3b] = Mdesign(4000, .05, 20, 20, 1.5, .25, 't1', [0,10,20], 'L', 'S', 'H', 3)

%% Figure 4:
%% [Frequencies4a, Measures4a] = Mdesign(4000, .85, 20, 20, 1.5, .25, 't1', [0:10:20], 'L', 'M', 'H', 3)
%% [Frequencies4b, Measures4b] = Mdesign(4000, .85, 20, 20, 1.5, .25, 't1', [0:10:20], 'L', 'S', 'H', 3)

%% Figure 5:
%% [Frequencies5a, Measures5a] = Mdesign(4000, [0:.25:1], 20, 20, 1.5, .25, 't1', 10, 'L', 'M', 'H', 1)
%% [Frequencies5b, Measures5b] = Mdesign(4000, [0:.25:1], 20, 20, 1.5, .25, 't1', 10, 'L', 'S', 'H', 1)

%% Figure 6:
%% [Frequencies6a, Measures6a] = Mdesign(4000, [.25, .5], 20, 20, 1.5, .15, 't1', 10,'C', 'M', 'H', 1)
%% [Frequencies6b, Measures6b] = Mdesign(4000, [.25, .5], 20, 20, 1.5, .15, 't1', 10,'C', 'S', 'H', 1)

%% Figure 7:
%% [Frequencies7a, Measures7a] = Mdesign(4000, .5, 20, 20, 5, .05, [1, 2, 3], 10, 'L', 'M', 'B', 5)
%% [Frequencies7b, Measures7b] = Mdesign(4000, .5, 20, 20, 5, .35, [1, 2, 3], 10, 'L', 'M', 'B', 5)

%% Figure 8:
%% [Frequencies8a, Measures8a] = Mdesign(4000, .05, 20, 20, 5, .25, [1, 2, 3], 10, 'L', 'M', 'B', 5)
%% [Frequencies8b, Measures8b] = Mdesign(4000, .85, 20, 20, 5, .25, [1, 2, 3], 10, 'L', 'M', 'B', 5)

%% Figure 9: (unstable; run repeatedly and choose the best)
%% [Frequencies9a, Measures9a] = Mdesign(4000, [.2, .8], 20, 20, 5, .25, 't1', 10, 'C', 'M', 'B', 1) 
%% [Frequencies9b, Measures9b] = Mdesign(4000, [.2, .8], 20, 20, 5, .25, 't1', 10, 'C', 'S', 'B', 1) 

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Frequencies, Measures] = Mdesign(trials, r, n, N, c, eps, df, k, Regtype, Mtype, psitype, VARY)
% Set this to adjust the ticks on the y-axes
tickspace = .10;

Moments = [];
Loss = [];
Frequencies = [];
Sigma = [];

if (VARY == 1)
    R = r;
    Length = length(R); 
elseif (VARY == 2)
    C = c;
    Length = length(C);
elseif (VARY == 3)
    K = k;
    Length = length(K);
elseif (VARY == 4)
    Eps = eps;
    Length = length(Eps);
elseif (VARY == 5)
    Df = df;
    Length = length(Df);
end
    
for counter = 1:Length                                                                  
   disp(['counter = ', num2str(counter)]);
    if (VARY == 1)
        r = R(counter);
    elseif (VARY == 2)
        c = C(counter);
    elseif (VARY == 3)
        k = K(counter);
    elseif (VARY == 4)
        eps = Eps(counter);
    elseif (VARY == 5)
        ddf = Df(counter);
        if (ddf ==1) 
            df = 't1';
        elseif (ddf == 2)
            df = 't3';
        elseif (ddf == 3)
            df = 'N9';
        end
    end
   [x, bestfrequencies, bestdesign, bestloss, bestmoments, sigma] = ...
       runone(trials, r, n, N, c, eps, df, k, Regtype, Mtype, psitype);
   Moments = [Moments, bestmoments'];
   Loss = [Loss, bestloss];
   Sigma = [Sigma, sigma];
   Frequencies = [Frequencies, bestfrequencies];
end
   
text = ['param  '; 'loss   ';'mu2    '; 'mu4    '; 'mu6    '; 'sigma  '];  
%%There must be the same  amount of space between each set of '       's,
                                                                                        
figure();
plotOnlyDesigns(x, Frequencies, Length, tickspace);
if (VARY == 1)
    Measures0 = [R; Loss; Moments; Sigma]; 
elseif (VARY == 2)
    Measures0 = [C; Loss; Moments; Sigma];
elseif (VARY == 3)
    Measures0 = [K; Loss; Moments; Sigma];
elseif (VARY == 4)
    Measures0 = [Eps; Loss; Moments; Sigma];
elseif (VARY == 5)
    Measures0 = [Df; Loss; Moments; Sigma];
end
Measures = [text num2str(Measures0)];
Frequencies = [Measures0(1,:); Frequencies];                                                        


%% ======================== subfunction "runone" ============================
function [x, bestfrequencies, bestdesign, bestloss, bestmoments, sigma] = ...
    runone(trials, r, n, N, c, eps, df, k, Regtype, Mtype, psitype)


%% Set the inital temperature
T = 1;

%% Set the design space
M = N/2;
x = (1:M)/M;
x = [sort(-x),x]';
w = weights(x,k);
indices = 1:N;
first_indices = indices(1:(N/2));

%% Set Z
Z = [ones(1,N);x'; x'.^2; x'.^3]';
if(Regtype == 'L')
    Z = Z(:, 1:2); % Straight line regression
elseif(Regtype == 'Q')
  Z = Z(:, 1:3); % Quadratic regression 
elseif(Regtype == 'C')
  Z = Z(:, 1:4); % Cubic regression  
end

%% Compute U
[U,S,V] = svd(Z,0);

%% Set design, hence p
chosen = randchoice(first_indices, n/2);
%% chosen = first_indices(1:n/2)    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
chosen = symmetrize(chosen, N);
design = zeros(N,1);
design(chosen) = 1;

p = design/n;
Design = design;

% Compute loss the first time

options = optimset('TolX',1e-7);
sigma = fzero(@(sigma) d(sigma, c, w, p, eps, df, Mtype, psitype), 1, options);
[d1 d2] = Calcd1d2(sigma, c, w, eps, df, Mtype, psitype);
lossout = loss(p, d1, d2, r, U);

Loss = lossout;
Sigma = sigma;
Accept = 1;

%%%%%%%%%%%% Annealing loop %%%%%%%%%%%%%%
for count = 1:trials
%    count
    currentloss = lossout;
    currentdesign = design;
    
    %Choose a positive p to reduce by 1/n and a p to increase by 1/n
    % and then symmetrize
    removed = randchoice(indices(design>0), 1);  
    removed = symmetrize(removed, N);
    
    added = randchoice(indices, 1); 
    added = symmetrize(added, N);    
 
    % Decrease the temperature every 100 iterations
    if(rem(count,100))==0
        T = .9*T;
    end
    
    design(added) = design(added) + 1;
    design(removed) = design(removed) - 1;
    
    p = design/n;
    
    % Recompute the loss
    if(Mtype == 'M')
        lossout = loss(p, d1, d2, r, U);
    elseif(Mtype == 'S')
        options = optimset('TolX',1e-7);    
        sigma = fzero(@(sigma) d(sigma, c, w, p, eps, df, Mtype, psitype), sigma, options);
        [d1 d2] = Calcd1d2(sigma, c, w, eps, df, Mtype, psitype); 
        lossout = loss(p, d1, d2, r, U);
    end
      
    trialloss = lossout;
    trialdesign = design;
   
    deltaloss = trialloss-currentloss; %Accept w.p.1 if deltaloss <0 
    accept = (exp(-deltaloss/T)>unifrnd(0,1));
    if(accept==1) 
        loss = trialloss;
        design = trialdesign;        
    else
        loss = currentloss;
        design = currentdesign;        
    end
Design = [Design,design];
Loss = [Loss, loss];
Sigma = [Sigma, sigma];
Accept = [Accept, accept];
end

indices = 1:(1+trials);
bestindex = indices(Loss == min(Loss));
bestindex = bestindex(1);
bestdesign = Design(:,bestindex);
bestfrequencies = bestdesign;
sigma = Sigma(bestindex);

bestdesign = [x(bestdesign>0), bestdesign(bestdesign>0)];
bestmoments = moments(bestdesign,n);
bestloss = Loss(bestindex);
%% Uncomment the next 2 lines to get plots after each step:
%pause(4);
%plotAnneal(x, Loss, bestdesign, bestloss);


%% ======================== subfunction "d" ============================
%% Sigma is the zero of this function, for fixed values of its other arguments
function d = d(sigma, c, w, p, eps, df, Mtype, psitype)


if(psitype == 'H')
    if(Mtype == 'M')
        [outPhi, outdf, outF] = H(c, df, eps);
        A = outPhi;
    
        cs = c*sigma;
        [outPhi, outdf, outF] = H(cs, df, eps);
        B = outPhi;
        C = outdf;
    
    elseif(Mtype == 'S')
        M = length(w)/2;
        w1 = w((M+1):(2*M));
        p1 = p((M+1):(2*M));
        cw = c*w1;
        [outPhi, outdf, outF] = H(cw, df, eps);
        A = w1.*w1.*p1.*outPhi;
        
        cs = cw*sigma;
        [outPhi, outdf, outF] = H(cs, df, eps);
        B = w1.*w1.*p1.*outPhi;
        C = w1.*w1.*p1.*outdf;
        A = sum(A);
        B = sum(B);
        C = sum(C);
    end
  
elseif(psitype == 'B')
     if(Mtype == 'M')
        [outPhi, outdf, outF] = B1(c, df, eps);
        A = outPhi;
    
        cs = c*sigma;
        [outPhi, outdf, outF] = B1(cs, df, eps);
        B = outPhi;
        C = outdf;
    
    elseif(Mtype == 'S')
        M = length(w)/2;
        w1 = w((M+1):(2*M));
        p1 = p((M+1):(2*M));
        cw = c*w1;
        [outPhi, outdf, outF] = B1(cw, df, eps);
        A = w1.*w1.*p1.*outPhi;
        
        cs = cw*sigma;
        [outPhi, outdf, outF] = B1(cs, df, eps);
        B = w1.*w1.*p1.*outPhi;
        C = w1.*w1.*p1.*outdf;
        A = sum(A);
        B = sum(B);
        C = sum(C);
    end
end

d = eps - (A-B)/(C-B);



%% ======================== subfunction "Calcd1d2" ============================
function [d1 d2] = Calcd1d2(sigma, c, w, eps, df, Mtype, psitype)  
M = length(w)/2;
w1 = w((M+1):(2*M));
if(psitype == 'H')    
    if(Mtype == 'M')
        cs = c*sigma;
        [outPhi, outdf, outF] = H(cs, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 1);
        elseif strcmp(df, 't3')            
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 3);
        elseif strcmp(df, 'N9')            
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*normcdf(-cs, 0, 3);
        end
        
        d1 = d1*w1;
        d2 = d2*w1.*w1;
        
    elseif(Mtype == 'S')
        cw = c*sigma*w1;
        [outPhi, outdf, outF] = H(cw, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 1);
        elseif strcmp(df, 't3')            
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 3);
        elseif strcmp(df, 'N9')            
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*normcdf(-cw, 0, 3);
        end
        d2 = d2.*w1.*w1;
    end
 
elseif(psitype == 'B')
    if(Mtype == 'M')
        cs = c*sigma;
        [outPhi, outdf, outF] = B3(cs, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 1) - 2*B2(cs, df, eps);
        elseif strcmp(df, 't3')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 3) - 2*B2(cs, df, eps);
        elseif strcmp(df, 'N9')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*normcdf(-cs, 0, 3) - 2*B2(cs, df, eps);                        
        end
        
        d1 = d1*w1;
        d2 = d2*w1.*w1;
        
    elseif(Mtype == 'S')
        cw = c*sigma*w1;
        [outPhi, outdf, outF] = B3(cw, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 1) - 2*B2(cw, df, eps);
        elseif strcmp(df, 't3')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 3) - 2*B2(cw, df, eps);
        elseif strcmp(df, 'N9')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*normcdf(-cw, 0, 3) - 2*B2(cw, df, eps);                        
        end
        d2 = d2.*w1.*w1;
    end
end

d1d2 = [d1 d2];
d1d2 = [flipud(d1d2); d1d2];
d1 = d1d2(:,1);
d2 = d1d2(:,2);

%% ======================== subfunction "symmetrize" ============================
function symmetrize = symmetrize(chosen, N)
symmetrize = sort([chosen,N+1-chosen]);

%% ======================== subfunction "loss" ============================
function out = loss(p, d1, d2, r, U)

D1 = diag(d1);
D2 = diag(d2);
P = diag(p);
M1 = U'*D1*P*U;
M2 = U'*D2*P*U;
M3 = U'*D1*P*P*D1*U;
M1inv = M1^-1;
M121 = M1inv*M2*M1inv';
L1 = trace(M121);
M131 = M1inv*M3*M1inv';
L2  = max(eig(M131));
out = r*L1 + (1-r)*L2;

%% ============================= subfunction "weights" ====================
function w = weights(x,k)
w=1./sqrt(1+k*x.^2);

%% ======================== subfunction "randchoice" ======================
function randchoice = randchoice(set, n)   %% Randomly choose n elements of a set
N = max(size(set));
qwe = randperm(N);
mypermute = set(qwe);
randchoice = mypermute(1:n);

%% ======================== subfunction "moments" ======================
function moments = moments(design, n)   %% Compute the first few moments
for q=1:3
    moments(q) = sum((design(:,1).^(2*q)).*(design(:,2)))/n;
end

%% ======================== subfunction "rep" ======================
%% Repeats the elements of X with frequencies in N
function rep = rep(X,N)
rep = [];
for i=1:length(N)
    rep=[rep; srep(X(i),N(i))];
end
 
%% ======================== subfunction "srep" ======================
%% Called by rep; does the same thing as rep if X and N have just one element each
function srep = srep(x,n) 
srep = x*ones(n,1);

%% ================== subfunction "plotAnneal" ======================

function plotAnneal(x, Loss, bestdesign, bestloss)
figure();
subplot(1, 2, 1); plot(1:length(Loss), Loss); xlabel('(a)'); 
    title('Accepted loss vs. trial');
    axis([1, length(Loss), min(Loss), max(Loss)]);
subplot(1, 2, 2); bar(bestdesign(:,1), bestdesign(:,2)); xlabel('(b)'); 
    axis([min(x)-.1, max(x)+.1, 0, max(bestdesign(:,2))]);
    title('Best Design');        

%% ================== subfunction "plotOnlyDesigns" ======================

function plotOnlyDesigns(x, Frequencies, Length, tickspace)
alphabet = ['(a)', '(b)', '(c)', '(d)', '(e)', '(f)', '(g)', '(h)', '(i)', '(j)', '(k)'];
Probs = Frequencies/sum(Frequencies(:,1));

for counter = 1:Length    
    subplot(1, Length, counter); 
    bar(x, Probs(:,counter)); 
    set(gca,'YTick',0:tickspace:.5);
    xlabel(alphabet((3*counter-2):(3*counter)));
    axis([min(x)-.1, max(x)+.1, 0, max(max(Probs(:,:)))]);    
end

%% ================== subfunction "Fint" ======================
function int = Fint(a,b, df) 
%% Computes the integral over [0, a] of (z^b)*density(z) where the
%% density is Cauchy ('t1') or Normal ('Normal')
%% Those for t1 are gotten from: syms x; a=int(x^b/(1+x^2),x)
if strcmp(df,'t1')
    if (b == 2)
        out = @(x) (x - atan(x));
    elseif (b == 4)
       out = @(x) (atan(x) - x + x.^3/3); 
    elseif (b == 6)
       out = @(x) (x - atan(x) - x.^3/3 + x.^5/5); 
	elseif (b == 8)
       out = @(x) (atan(x) - x + x.^3/3 - x.^5/5 + x.^7/7); 
	elseif (b == 10)
       out = @(x) (x - atan(x) - x.^3/3 + x.^5/5 - x.^7/7 + x.^9/9); 
    end
    int = out(a)/pi;
elseif strcmp(df,'t3')
    if (b == 2)
        out = @(x) ((3*3^(1/2)*atan((3^(1/2)*x)/3))/2 - (9*x)./(2*(x.^2 + 3)));
    elseif (b == 4)
       out = @(x) (9*x - (27*3^(1/2)*atan((3^(1/2)*x)/3))/2 + (27*x)./(2*(x.^2 + 3))); 
    elseif (b == 6)
       out = @(x) ((135*3^(1/2)*atan((3^(1/2)*x)/3))/2 - 54*x - (81*x)./(2*(x.^2 + 3)) + 3*x.^3); 
	elseif (b == 8)
       out = @(x) (243*x - (567*3^(1/2)*atan((3^(1/2)*x)/3))/2 + (243*x)./(2*(x.^2 + 3)) - 18*x.^3 + (9*x.^5)/5); 
	elseif (b == 10)
       out = @(x) ((2187*3^(1/2)*atan((3^(1/2)*x)/3))/2 - 972*x - (729*x)./(2*(x.^2 + 3)) + 81*x.^3 - (54*x.^5)/5 + (9*x.^7)/7); 
    end
    int = 2*out(a)/(pi*sqrt(3));    
elseif strcmp(df, 'Normal') 
    out = @(x)(2^(b/2-1))*gamma((b+1)/2)*gamcdf((x.^2)/2, (b+1)/2)/sqrt(pi);
    int = out(a);
elseif strcmp(df, 'N9') 
    out = @(x)(2^(b/2-1))*gamma((b+1)/2)*gamcdf((x.^2)/2, (b+1)/2)/sqrt(pi);
    int = (3^b)*out(a/3);
end

%% ================== subfunction "B1" ======================
function [outPhi, outdf, outF] = B1(c, df, eps)
outPhi = Fint(c, 2, 'Normal')./(c.^2) - 3*Fint(c, 4, 'Normal')./(c.^4) + ...
    5*Fint(c,6, 'Normal')./(3*(c.^6));
outdf = Fint(c, 2, df)./(c.^2) - 3*Fint(c, 4, df)./(c.^4) + ...
    5*Fint(c,6, df)./(3*(c.^6));
outF = (1 - eps)*outPhi + eps*outdf;

%% ================== subfunction "B2" ======================
function [outPhi, outdf, outF] = B2(c, df, eps)
outPhi = 6*Fint(c, 2, 'Normal')./(c.^2) - 5*Fint(c, 4, 'Normal')./(c.^4);
outdf = 6*Fint(c, 2, df)./(c.^2) - 5*Fint(c, 4, df)./(c.^4);
outF = (1 - eps)*outPhi + eps*outdf;

%% ================== subfunction "B3" ======================
function [outPhi, outdf, outF] = B3(c, df, eps)
outPhi = Fint(c, 2, 'Normal')./(c.^2) - 4*Fint(c, 4, 'Normal')./(c.^4) + ...
    6*Fint(c,6, 'Normal')./(c.^6) - 4*Fint(c,8, 'Normal')./(c.^8) + ...
    Fint(c,10, 'Normal')./(c.^10);
outdf = Fint(c, 2, df)./(c.^2) - 4*Fint(c, 4, df)./(c.^4) + ...
    6*Fint(c,6, df)./(c.^6) - 4*Fint(c,8, df)./(c.^8) + ...
    Fint(c,10, df)./(c.^10);
outF = (1 - eps)*outPhi + eps*outdf;

%% ================== subfunction "H" ======================
function [outPhi, outdf, outF] = H(c, df, eps)
outPhi = Fint(c, 2, 'Normal')./(c.^2) + normcdf(-c);
if strcmp(df,'t1')
    outdf = Fint(c, 2, 't1')./(c.^2) + tcdf(-c,1);
elseif strcmp(df,'t3')
    outdf = Fint(c, 2, 't3')./(c.^2) + tcdf(-c,3);
elseif strcmp(df,'N9')
    outdf = Fint(c, 2, 'N9')./(c.^2) + normcdf(-c,0,3);
end
outF = (1 - eps)*outPhi + eps*outdf;



