% For Figures 10 to 13
function [finaldesign, Measures] = Mdesign2(trials, r, c, eps, df, k, Mtype, psitype)

%% Input:
% trials must be a multiple of 100
% r; r=0 for pure bias, r=1 for pure variance
% c; tuning parameter of psitype 
% eps = proportion of contaminated errors
% df: distribution of the contamination; 't1' (Cauchy)
% k; tuning parameter in the weights
% Mtype = Mallows ('M') or Schweppe ('S') 
% psitype = Huber ('H') or bisquare ('B')

%% Figures 10 to 13 are plotted by SPlus with data obtained as in the following:
%% Figure 10(a)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 1.5, 0.15, 't1', 0, 'M', 'H')
%% Figure 10(b)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 1.5, 0.15, 't1', 10, 'M', 'H')
%% Figure 10(c)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 1.5, 0.15, 't1', 20, 'M', 'H')
%% Figure 11(a)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 5, 0.15, 't1', 0, 'M', 'B')
%% Figure 11(b)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 5, 0.15, 't1', 10, 'M', 'B')
%% Figure 11(c)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 5, 0.15, 't1', 20, 'M', 'B')
%% Figure 12(a)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 1.5, 0.15, 't1', 0, 'S', 'H')
%% Figure 12(b)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 1.5, 0.15, 't1', 10, 'S', 'H')
%% Figure 12(c)
%[[finaldesign, Measures] = Mdesign2(4000, 0.15, 1.5, 0.15, 't1', 20, 'S', 'H')
%% Figure 13(a)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 5, 0.15, 't1', 0, 'S', 'B')
%% Figure 13(b)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 5, 0.15, 't1', 10, 'S', 'B')
%% Figure 13(c)
%[finaldesign, Measures] = Mdesign2(4000, 0.15, 5, 0.15, 't1', 20, 'S', 'B')
% 
% Remark: % There are n = 80 selected points and Nsupport = 120 possible support points)

Moments = [];
Loss = [];
Frequencies = [];
Sigma = [];
    
   [x, bestfrequencies, bestdesign, finaldesign, bestloss, bestmoments, sigma] = ...
       runone(trials, r, c, eps, df, k, Mtype, psitype);
   
   Moments = [Moments, bestmoments'];
   Loss = [Loss, bestloss];
   Sigma = [Sigma, sigma];
   Frequencies = [Frequencies, bestfrequencies];

text = ['k      '; 'loss   '; 'sigma  ']; 
Measures = [k; Loss; Sigma];
Measures = [text num2str(Measures)];

%% ======================== subfunction "runone" ============================
function [x, bestfrequencies, bestdesign, finaldesign, bestloss, bestmoments, sigma] = runone(trials, r, c, eps, df, k, Mtype, psitype);


%% Set the inital temperature
T = 1;

%% Set the design space
%------------------------------------------
N=144;
n = 80;
%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nsupport1= 121;
Nsupport = 120;
M1= sqrt(N)/2;
%%%%%%%%%%%%%%%%%%%%%%%
x = linspace(-1,1,120)';
w = weights(x, k);
     x2=[-1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1];
     x2=[x2,x2+0.2,x2+0.4,x2+0.6,x2+0.8,x2+1,x2+1.2,x2+1.4,x2+1.6,x2+1.8,x2+2];
     x1=[-1 -0.8 -0.6 -0.4 -0.2 0 0.2 0.4 0.6 0.8 1];
     x1=[x1,x1,x1,x1,x1,x1,x1,x1,x1,x1,x1];

     indices = 1:Nsupport;
     Nrandom = 20;
     iindices = 1:Nrandom;
first_indices = iindices(1:Nrandom);
%% Set Z
Z = [ones(1,Nsupport1);x1;x2]';
%---------------------------------------
Z(61,:)=[];

%% Compute U
[U,S,V] = svd(Z,0);
[NN,pp] = size(U);

%% Set design, hence p
%            chosen = randchoice(first_indices, n);
     n8 = n/8;
cchosen = randchoice(first_indices, n8);
design = zeros(Nsupport,1);
        for iii = 1:n8
        if cchosen(iii) == 1
        design(1)=design(1)+2;
        design(11)=design(11)+2;
        design(110)=design(110)+2;
        design(120)=design(120)+2; 
        elseif cchosen(iii) == 2
        design(2)=design(2)+1;
        design(10)=design(10)+1;
        design(12)=design(12)+1;
        design(22)=design(22)+1;
        design(99)=design(99)+1;
        design(109)=design(109)+1;
        design(111)=design(111)+1;
        design(119)=design(119)+1;      
        elseif cchosen(iii) == 3
        design(3)=design(3)+1;
        design(9)=design(9)+1;
        design(23)=design(23)+1;
        design(33)=design(33)+1;
        design(88)=design(88)+1;
        design(98)=design(98)+1;
        design(112)=design(112)+1;
        design(118)=design(118)+1;   
        elseif cchosen(iii) == 4
        design(4)=design(4)+1;
        design(8)=design(8)+1;
        design(34)=design(34)+1;
        design(44)=design(44)+1;
        design(77)=design(77)+1;
        design(87)=design(87)+1;
        design(113)=design(113)+1;
        design(117)=design(117)+1;     
        elseif cchosen(iii) == 5
        design(5)=design(5)+1;
        design(7)=design(7)+1;
        design(45)=design(45)+1;
        design(55)=design(55)+1;
        design(66)=design(66)+1;
        design(76)=design(76)+1;
        design(114)=design(114)+1;
        design(116)=design(116)+1;            
        elseif cchosen(iii) == 6
        design(6)=design(6)+2;
        design(56)=design(56)+2;
        design(65)=design(65)+2;
        design(115)=design(115)+2;
        elseif cchosen(iii) == 7
        design(13)=design(13)+2;
        design(21)=design(21)+2;
        design(100)=design(100)+2;
        design(108)=design(108)+2;
        elseif cchosen(iii) == 8
        design(14)=design(14)+1;
        design(20)=design(20)+1;
        design(24)=design(24)+1;
        design(32)=design(32)+1;
        design(89)=design(89)+1;
        design(97)=design(97)+1;
        design(101)=design(101)+1;
        design(107)=design(107)+1;  
        elseif cchosen(iii) == 9
        design(15)=design(15)+1;
        design(19)=design(19)+1;
        design(35)=design(35)+1;
        design(43)=design(43)+1;
        design(78)=design(78)+1;
        design(86)=design(86)+1;
        design(102)=design(102)+1;
        design(106)=design(106)+1;    
        elseif cchosen(iii) == 10
        design(16)=design(16)+1;
        design(18)=design(18)+1;
        design(46)=design(46)+1;
        design(54)=design(54)+1;
        design(67)=design(67)+1;
        design(75)=design(75)+1;
        design(103)=design(103)+1;
        design(105)=design(105)+1;        
        elseif cchosen(iii) == 11
        design(17)=design(17)+2;
        design(57)=design(57)+2;
        design(64)=design(64)+2;
        design(104)=design(104)+2;
        elseif cchosen(iii) == 12
        design(25)=design(25)+2;
        design(31)=design(31)+2;
        design(90)=design(90)+2;
        design(96)=design(96)+2;
        elseif cchosen(iii) == 13
        design(26)=design(26)+1;
        design(30)=design(30)+1;
        design(36)=design(36)+1;
        design(42)=design(42)+1;
        design(79)=design(79)+1;
        design(85)=design(85)+1;
        design(91)=design(91)+1;
        design(95)=design(95)+1;                
        elseif cchosen(iii) == 14
        design(27)=design(27)+1;
        design(29)=design(29)+1;
        design(47)=design(47)+1;
        design(53)=design(53)+1;
        design(68)=design(68)+1;
        design(74)=design(74)+1;
        design(92)=design(92)+1;
        design(94)=design(94)+1;         
        elseif cchosen(iii) == 15
        design(28)=design(28)+2;
        design(58)=design(58)+2;
        design(63)=design(63)+2;
        design(93)=design(93)+2;
        elseif cchosen(iii) == 16
        design(37)=design(37)+2;
        design(41)=design(41)+2;
        design(80)=design(80)+2;
        design(84)=design(84)+2;
        elseif cchosen(iii) == 17
        design(38)=design(38)+1;
        design(40)=design(40)+1;
        design(48)=design(48)+1;
        design(52)=design(52)+1;
        design(81)=design(81)+1;
        design(83)=design(83)+1;
        design(69)=design(69)+1;
        design(73)=design(73)+1; 
        elseif cchosen(iii) == 18
        design(39)=design(39)+2;
        design(59)=design(59)+2;
        design(62)=design(62)+2;
        design(82)=design(82)+2;
        elseif cchosen(iii) == 19
        design(49)=design(49)+2;
        design(51)=design(51)+2;
        design(70)=design(70)+2;
        design(72)=design(72)+2;
        elseif cchosen(iii) == 20
        design(50)=design(50)+2;
        design(60)=design(60)+2;
        design(61)=design(61)+2;
        design(71)=design(71)+2;
        end
        end
       
%design(cchosen) = 1;
 p = design/n;
% lengthp=length(p)
Design = design;

% Compute loss the first time
options = optimset('TolX',1e-7);
sigma = fzero(@(sigma) d(sigma, c, w, p, eps, df, Mtype, psitype), 1, options);
[d1 d2] = Calcd1d2(sigma, c, w, eps, df, Mtype, psitype);
lossout = loss(p, d1, d2, r, U);

Loss = lossout;
Sigma = sigma;
Accept = 1;

%%%%%%%%%%%% Annealing loop %%%%%%%%%%%%%%
    for count = 1:trials
    currentloss = lossout;
    currentdesign = design;
%    currentM1inv = M1inv;
    
    removed = randchoice(indices(design>0), 1);  
    if removed == 1 | 11 | 110 | 120
    rem1=1;
    elseif removed == 2 | 10 | 12 | 22 | 99 | 109 | 111 | 119
    rem1=2;
    elseif  removed == 3 | 9 | 23 | 33 | 88 | 98 | 112 | 118
    rem1=3;
    elseif  removed == 4 | 8 | 34 | 44 | 77 | 87 | 113 | 117
    rem1=4;
    elseif  removed == 5 | 7 | 45 | 55 | 66 | 76 | 114 | 116
    rem1=5;
    elseif  removed == 6 | 56 | 65 | 115
    rem1=6;
    elseif  removed == 13 | 21 | 100 | 108
    rem1=7;
    elseif  removed == 14 | 20 | 24 | 32 | 89 | 97 | 101 | 107
    rem1=8;
    elseif  removed == 15 | 19 | 35 | 43 | 78 | 86 | 102 | 106
    rem1=9;
    elseif  removed == 16 | 18 | 46 | 54 | 67 | 75 | 103 | 105
    rem1=10;
    elseif  removed == 17 | 57 | 64 | 104
    rem1=11;
    elseif  removed == 25 | 31 | 90 | 96
    rem1=12;
    elseif  removed == 26 | 30 | 36 | 42 | 79 | 85 | 91 | 95
    rem1=13;
    elseif  removed == 27 | 29 | 47 | 53 | 68 | 74 | 92 | 94
    rem1=14;
    elseif  removed == 28 | 58 | 63 | 93
    rem1=15;
    elseif  removed == 37 | 41 | 80 | 84
    rem1=16;
    elseif  removed == 38 | 40 | 48 | 52 | 81 | 83 | 69 | 73
    rem1=17;
    elseif  removed == 39 | 59 | 62 | 82
    rem1=18;
    elseif  removed == 49 | 51 | 70 | 72
    rem1=19;
    elseif  removed == 50 | 60 | 61 | 71
    rem1=20;
    end
    done = 0;
    p = design/n;
        if (rem1 == 1)
        if (design(1) > 1)
        design(1)=design(1)-2;
        design(11)=design(11)-2;
        design(110)=design(110)-2;
        design(120)=design(120)-2; 
        done = 1;
        end
        elseif  (rem1 == 2)
        if (design(2) > 0)
        design(2)=design(2)-1;
        design(10)=design(10)-1;
        design(12)=design(12)-1;
        design(22)=design(22)-1;
        design(99)=design(99)-1;
        design(109)=design(109)-1;
        design(111)=design(111)-1;
        design(119)=design(119)-1;      
        done = 1;
        end
        elseif  (rem1 == 3)
        if (design(3) > 0)            
        design(3)=design(3)-1;
        design(9)=design(9)-1;
        design(23)=design(23)-1;
        design(33)=design(33)-1;
        design(88)=design(88)-1;
        design(98)=design(98)-1;
        design(112)=design(112)-1;
        design(118)=design(118)-1;   
        done = 1;
        end
        elseif  (rem1 == 4)
        if (design(4) > 0)
        design(4)=design(4)-1;
        design(8)=design(8)-1;
        design(34)=design(34)-1;
        design(44)=design(44)-1;
        design(77)=design(77)-1;
        design(87)=design(87)-1;
        design(113)=design(113)-1;
        design(117)=design(117)-1;     
        done = 1;
        end
        elseif  (rem1 == 5)
        if (design(5) > 0)
        design(5)=design(5)-1;
        design(7)=design(7)-1;
        design(45)=design(45)-1;
        design(55)=design(55)-1;
        design(66)=design(66)-1;
        design(76)=design(76)-1;
        design(114)=design(114)-1;
        design(116)=design(116)-1;            
        done = 1;
        end
        elseif  (rem1 == 6)
        if (design(6) > 1)
        design(6)=design(6)-2;
        design(56)=design(56)-2;
        design(65)=design(65)-2;
        design(115)=design(115)-2;
        done = 1;
        end
        elseif  (rem1 == 7)
        if (design(13) > 1)
        design(13)=design(13)-2;
        design(21)=design(21)-2;
        design(100)=design(100)-2;
        design(108)=design(108)-2;
        done = 1;
        end
        elseif  (rem1 == 8)
        if (design(14) > 0)
        design(14)=design(14)-1;
        design(20)=design(20)-1;
        design(24)=design(24)-1;
        design(32)=design(32)-1;
        design(89)=design(89)-1;
        design(97)=design(97)-1;
        design(101)=design(101)-1;
        design(107)=design(107)-1;  
        done = 1;
        end
        elseif  (rem1 == 9)
        if (design(15) > 0)
        design(15)=design(15)-1;
        design(19)=design(19)-1;
        design(35)=design(35)-1;
        design(43)=design(43)-1;
        design(78)=design(78)-1;
        design(86)=design(86)-1;
        design(102)=design(102)-1;
        design(106)=design(106)-1;    
        done = 1;
        end
        elseif  (rem1 == 10)
        if (design(16) > 0)
        design(16)=design(16)-1;
        design(18)=design(18)-1;
        design(46)=design(46)-1;
        design(54)=design(54)-1;
        design(67)=design(67)-1;
        design(75)=design(75)-1;
        design(103)=design(103)-1;
        design(105)=design(105)-1;        
        done = 1;
        end
        elseif  (rem1 == 11)
        if (design(17) > 1)
        design(17)=design(17)-2;
        design(57)=design(57)-2;
        design(64)=design(64)-2;
        design(104)=design(104)-2;
        done = 1;
        end
        elseif  (rem1 == 12)
        if (design(25) > 1)
        design(25)=design(25)-2;
        design(31)=design(31)-2;
        design(90)=design(90)-2;
        design(96)=design(96)-2;
        done = 1;
        end
        elseif  (rem1 == 13)
        if (design(26) > 0)
        design(26)=design(26)-1;
        design(30)=design(30)-1;
        design(36)=design(36)-1;
        design(42)=design(42)-1;
        design(79)=design(79)-1;
        design(85)=design(85)-1;
        design(91)=design(91)-1;
        design(95)=design(95)-1;                
        done = 1;
        end
        elseif  (rem1 == 14)
        if (design(27) > 0)
        design(27)=design(27)-1;
        design(29)=design(29)-1;
        design(47)=design(47)-1;
        design(53)=design(53)-1;
        design(68)=design(68)-1;
        design(74)=design(74)-1;
        design(92)=design(92)-1;
        design(94)=design(94)-1;         
        done = 1;
        end
        elseif  (rem1 == 15)
        if (design(28) > 1)
        design(28)=design(28)-2;
        design(58)=design(58)-2;
        design(63)=design(63)-2;
        design(93)=design(93)-2;
        done = 1;
        end
        elseif  (rem1 == 16)
        if (design(37) > 1)
        design(37)=design(37)-2;
        design(41)=design(41)-2;
        design(80)=design(80)-2;
        design(84)=design(84)-2;
        done = 1;
        end
        elseif  (rem1 == 17)
        if (design(38) > 0)
        design(38)=design(38)-1;
        design(40)=design(40)-1;
        design(48)=design(48)-1;
        design(52)=design(52)-1;
        design(81)=design(81)-1;
        design(83)=design(83)-1;
        design(69)=design(69)-1;
        design(73)=design(73)-1; 
        done = 1;
        end
        elseif  (rem1 == 18)
        if (design(39) > 1)
        design(39)=design(39)-2;
        design(59)=design(59)-2;
        design(62)=design(62)-2;
        design(82)=design(82)-2;
        done = 1;
        end
        elseif  (rem1 == 19)
        if (design(49) > 1)
        design(49)=design(49)-2;
        design(51)=design(51)-2;
        design(70)=design(70)-2;
        design(72)=design(72)-2;
        done = 1;
        end
        elseif  (rem1 == 20)
        if (design(50) > 1)
        design(50)=design(50)-2;
        design(60)=design(60)-2;
        design(61)=design(61)-2;
        design(71)=design(71)-2;
        done = 1;
        end
    end
 added = randperm(20);  
       add1 = added;
    if (done == 1)
        if (add1 == 1)
        design(1)=design(1)+2;
        design(11)=design(11)+2;
        design(110)=design(110)+2;
        design(120)=design(120)+2; 
        elseif add1 == 2
        design(2)=design(2)+1;
        design(10)=design(10)+1;
        design(12)=design(12)+1;
        design(22)=design(22)+1;
        design(99)=design(99)+1;
        design(109)=design(109)+1;
        design(111)=design(111)+1;
        design(119)=design(119)+1;      
        elseif add1 == 3
        design(3)=design(3)+1;
        design(9)=design(9)+1;
        design(23)=design(23)+1;
        design(33)=design(33)+1;
        design(88)=design(88)+1;
        design(98)=design(98)+1;
        design(112)=design(112)+1;
        design(118)=design(118)+1;      
        elseif add1 == 4
        design(4)=design(4)+1;
        design(8)=design(8)+1;
        design(34)=design(34)+1;
        design(44)=design(44)+1;
        design(77)=design(77)+1;
        design(87)=design(87)+1;
        design(113)=design(113)+1;
        design(117)=design(117)+1;     
        elseif add1 == 5
        design(5)=design(5)+1;
        design(7)=design(7)+1;
        design(45)=design(45)+1;
        design(55)=design(55)+1;
        design(66)=design(66)+1;
        design(76)=design(76)+1;
        design(114)=design(114)+1;
        design(116)=design(116)+1;            
        elseifadd1 == 6
        design(6)=design(6)+2;
        design(56)=design(56)+2;
        design(65)=design(65)+2;
        design(115)=design(115)+2;
        elseif add1 == 7
        design(13)=design(13)+2;
        design(21)=design(21)+2;
        design(100)=design(100)+2;
        design(108)=design(108)+2;
        elseif add1 == 8
        design(14)=design(14)+1;
        design(20)=design(20)+1;
        design(24)=design(24)+1;
        design(32)=design(32)+1;
        design(89)=design(89)+1;
        design(97)=design(97)+1;
        design(101)=design(101)+1;
        design(107)=design(107)+1;  
        elseif add1 == 9
        design(15)=design(15)+1;
        design(19)=design(19)+1;
        design(35)=design(35)+1;
        design(43)=design(43)+1;
        design(78)=design(78)+1;
        design(86)=design(86)+1;
        design(102)=design(102)+1;
        design(106)=design(106)+1;    
        elseif add1 == 10
        design(16)=design(16)+1;
        design(18)=design(18)+1;
        design(46)=design(46)+1;
        design(54)=design(54)+1;
        design(67)=design(67)+1;
        design(75)=design(75)+1;
        design(103)=design(103)+1;
        design(105)=design(105)+1;        
        elseif add1 == 11
        design(17)=design(17)+2;
        design(57)=design(57)+2;
        design(64)=design(64)+2;
        design(104)=design(104)+2;
        elseif add1 == 12
        design(25)=design(25)+2;
        design(31)=design(31)+2;
        design(90)=design(90)+2;
        design(96)=design(96)+2;
        elseif add1 == 13
        design(26)=design(26)+1;
        design(30)=design(30)+1;
        design(36)=design(36)+1;
        design(42)=design(42)+1;
        design(79)=design(79)+1;
        design(85)=design(85)+1;
        design(91)=design(91)+1;
        design(95)=design(95)+1;                
        elseif add1 == 14
        design(27)=design(27)+1;
        design(29)=design(29)+1;
        design(47)=design(47)+1;
        design(53)=design(53)+1;
        design(68)=design(68)+1;
        design(74)=design(74)+1;
        design(92)=design(92)+1;
        design(94)=design(94)+1;         
        elseif add1 == 15
        design(28)=design(28)+2;
        design(58)=design(58)+2;
        design(63)=design(63)+2;
        design(93)=design(93)+2;
        elseif add1 == 16
        design(37)=design(37)+2;
        design(41)=design(41)+2;
        design(80)=design(80)+2;
        design(84)=design(84)+2;
        elseif add1 == 17
        design(38)=design(38)+1;
        design(40)=design(40)+1;
        design(48)=design(48)+1;
        design(52)=design(52)+1;
        design(81)=design(81)+1;
        design(83)=design(83)+1;
        design(69)=design(69)+1;
        design(73)=design(73)+1; 
        elseif add1 == 18
        design(39)=design(39)+2;
        design(59)=design(59)+2;
        design(62)=design(62)+2;
        design(82)=design(82)+2;
        elseif add1 == 19
        design(49)=design(49)+2;
        design(51)=design(51)+2;
        design(70)=design(70)+2;
        design(72)=design(72)+2;
        elseif add1 == 20
        design(50)=design(50)+2;
        design(60)=design(60)+2;
        design(61)=design(61)+2;
        design(71)=design(71)+2;
        end
        end
        p = design/n;
             
% Decrease the temperature every 100 iterations
  
    if(rem(count,100))==0
        T = .9*T;
    end

      % Recompute the loss
    if(Mtype == 'M')
        lossout = loss(p, d1, d2, r, U);
    elseif(Mtype == 'S')
        options = optimset('TolX',1e-7);    
        sigma = fzero(@(sigma) d(sigma, c, w, p, eps, df, Mtype, psitype), sigma, options);
        [d1 d2] = Calcd1d2(sigma, c, w, eps, df, Mtype, psitype); 
        lossout = loss(p, d1, d2, r, U);
    end
      
    trialloss = lossout;
    trialdesign = design;
   
    deltaloss = trialloss-currentloss;
    %Accept w.p.1 if deltaloss <0 
    accept = (exp(-deltaloss/T)>unifrnd(0,1));
    if(accept==1) 
        loss = trialloss;
        design = trialdesign;        
    else
        loss = currentloss;
        design = currentdesign;        
    end
Design = [Design,design];
Loss = [Loss, loss];
Sigma = [Sigma, sigma];
Accept = [Accept, accept];
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%end

indices = 1:(1+trials);
bestindex = indices(Loss == min(Loss));
bestindex = bestindex(1);
bestdesign = Design(:,bestindex);
bestfrequencies = bestdesign;
sigma = Sigma(bestindex);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bestdesign = [x(bestdesign>0), bestdesign(bestdesign>0)];
bestmoments = moments(bestdesign,n);
bestloss = Loss(bestindex);
%%%acceptprobs = mean(reshape(Accept(2:length(Accept)), 100, floor(trials/100)));
%pause(4);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%&&&&&&&&&&&&&&&&&&&&&&&&&
bestfrequencies = bestfrequencies/n;
finaldesign = [Z(:,2), Z(:,3), bestfrequencies];
%bestdesignpoints=[x1', x2', bestfrequencies]
%%%%%%%%%%%%%%%%%%%%%%%%%%%%&&&&&&&&&&&&&&&&&&&&&&&&&

    
%% ======================== subfunction "d" ============================
%% Sigma is the zero of this function, for fixed values of its other arguments
function d = d(sigma, c, w, p, eps, df, Mtype, psitype)


if(psitype == 'H')
    if(Mtype == 'M')
        [outPhi, outdf, outF] = H(c, df, eps);
        A = outPhi;
    
        cs = c*sigma;
        [outPhi, outdf, outF] = H(cs, df, eps);
        B = outPhi;
        C = outdf;
    
    elseif(Mtype == 'S')
        M = length(w)/2;
        w1 = w((M+1):(2*M));
        p1 = p((M+1):(2*M));
        cw = c*w1;
        [outPhi, outdf, outF] = H(cw, df, eps);
        A = w1.*w1.*p1.*outPhi;
        
        cs = cw*sigma;
        [outPhi, outdf, outF] = H(cs, df, eps);
        B = w1.*w1.*p1.*outPhi;
        C = w1.*w1.*p1.*outdf;
        A = sum(A);
        B = sum(B);
        C = sum(C);
    end
  
elseif(psitype == 'B')
     if(Mtype == 'M')
        [outPhi, outdf, outF] = B1(c, df, eps);
        A = outPhi;
    
        cs = c*sigma;
        [outPhi, outdf, outF] = B1(cs, df, eps);
        B = outPhi;
        C = outdf;
    
    elseif(Mtype == 'S')
        M = length(w)/2;
        w1 = w((M+1):(2*M));
        p1 = p((M+1):(2*M));
        cw = c*w1;
        [outPhi, outdf, outF] = B1(cw, df, eps);
        A = w1.*w1.*p1.*outPhi;
        
        cs = cw*sigma;
        [outPhi, outdf, outF] = B1(cs, df, eps);
        B = w1.*w1.*p1.*outPhi;
        C = w1.*w1.*p1.*outdf;
        A = sum(A);
        B = sum(B);
        C = sum(C);
    end
end

d = eps - (A-B)/(C-B);

%% ======================== subfunction "Calcd1d2" ============================
function [d1 d2] = Calcd1d2(sigma, c, w, eps, df, Mtype, psitype)  
M = length(w)/2;
w1 = w((M+1):(2*M));
if(psitype == 'H')    
    if(Mtype == 'M')
        cs = c*sigma;
        [outPhi, outdf, outF] = H(cs, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 1);
        elseif strcmp(df, 't3')            
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 3);
        elseif strcmp(df, 'N9')            
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*normcdf(-cs, 0, 3);
        end
        
        d1 = d1*w1;
        d2 = d2*w1.*w1;
        
    elseif(Mtype == 'S')
        cw = c*sigma*w1;
        [outPhi, outdf, outF] = H(cw, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 1);
        elseif strcmp(df, 't3')            
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 3);
        elseif strcmp(df, 'N9')            
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*normcdf(-cw, 0, 3);
        end
        d2 = d2.*w1.*w1;
    end
 
elseif(psitype == 'B')
    if(Mtype == 'M')
        cs = c*sigma;
        [outPhi, outdf, outF] = B3(cs, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 1) - 2*B2(cs, df, eps);
        elseif strcmp(df, 't3')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*tcdf(-cs, 3) - 2*B2(cs, df, eps);
        elseif strcmp(df, 'N9')
            d1 = 1-2*(1-eps)*normcdf(-cs) - 2*eps*normcdf(-cs, 0, 3) - 2*B2(cs, df, eps);                        
        end
        
        d1 = d1*w1;
        d2 = d2*w1.*w1;
        
    elseif(Mtype == 'S')
        cw = c*sigma*w1;
        [outPhi, outdf, outF] = B3(cw, df, eps);
        d2 = 2*c^2*outF;
        if strcmp(df,'t1')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 1) - 2*B2(cw, df, eps);
        elseif strcmp(df, 't3')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*tcdf(-cw, 3) - 2*B2(cw, df, eps);
        elseif strcmp(df, 'N9')
            d1 = 1-2*(1-eps)*normcdf(-cw) - 2*eps*normcdf(-cw, 0, 3) - 2*B2(cw, df, eps);                        
        end
        d2 = d2.*w1.*w1;
    end
end

d1d2 = [d1 d2];
d1d2 = [flipud(d1d2); d1d2];
d1 = d1d2(:,1);
d2 = d1d2(:,2);


%% ======================== subfunction "loss" ============================
function out = loss(p, d1, d2, r, U)

D1 = diag(d1);
D2 = diag(d2);
P = diag(p);
M1 = U'*D1*P*U;
M2 = U'*D2*P*U;
M3 = U'*D1*P*P*D1*U;
M1inv = M1^-1;
M121 = M1inv*M2*M1inv';
L1 = trace(M121);
M131 = M1inv*M3*M1inv';
L2  = max(eig(M131));
out = r*L1 + (1-r)*L2;

%% ============================= subfunction "weights" ====================
function w = weights(x,k)
y = x ;
y(1) = 1;
y(2) = 0.8;
y(3) = 0.6;
y(4) = 0.4;
y(5) = 0.2;
y(6) = 0.0;
y(13)= sqrt(0.8^2+0.2^2);
y(14)= sqrt(0.6^2+0.2^2);
y(15)= sqrt(0.4^2+0.2^2);
y(16)= sqrt(0.2^2+0.2^2);
y(17)= 0.2;
y(25)= sqrt(0.6^2+0.4^2);
y(26)= sqrt(0.4^2+0.4^2);
y(27)= sqrt(0.2^2+0.4^2);
y(28)= 0.4;
y(37)= sqrt(0.4^2+0.6^2);
y(38)= sqrt(0.2^2+0.6^2);
y(39)= 0.6;
y(49)= sqrt(0.2^2+0.8^2);
y(50)= 0.8;
y(11) = y(1);
y(110) = y(1);
y(120) = y(1);
y(10) = y(2);
y(12) = y(2);
y(22) = y(2);
y(99) = y(2);
y(109) = y(2);
y(111) = y(2);
y(119) = y(2);
y(9) = y(3);
y(23) = y(3);
y(33) = y(3);
y(88) = y(3);
y(98) = y(3);
y(112) = y(3);
y(118) = y(3);
y(8) = y(4);
y(34) = y(4);
y(44) = y(4);
y(77) = y(4);
y(87) = y(4);
y(113) = y(4);
y(117) = y(4);
y(7) = y(5);
y(45) = y(5);
y(55) = y(5);
y(66) = y(5);
y(76) = y(5);
y(114) = y(5);
y(116) = y(5);
y(56) = y(6);
y(65) = y(6);
y(115) = y(6);
y(21) = y(13);
y(100) = y(13);
y(108) = y(13);
y(20) = y(14);
y(24) = y(14);
y(32) = y(14);
y(89) = y(14);
y(97) = y(14);
y(101) = y(14);
y(107) = y(14);
y(19) = y(15);
y(35) = y(15);
y(43) = y(15);
y(78) = y(15);
y(86) = y(15);
y(102) = y(15);
y(106) = y(15);
y(18) = y(16);
y(46) = y(16);
y(54) = y(16);
y(67) = y(16);
y(75) = y(16);
y(103) = y(16);
y(105) = y(16);
y(57) = y(17);
y(64) = y(17);
y(104) = y(17);
y(31) = y(25);
y(90) = y(25);
y(96) = y(25);
y(30) = y(26);
y(36) = y(26);
y(42) = y(26);
y(79) = y(26);
y(85) = y(26);
y(91) = y(26);
y(95) = y(26);
y(29) = y(27);
y(47) = y(27);
y(53) = y(27);
y(68) = y(27);
y(74) = y(27);
y(92) = y(27);
y(94) = y(27);
y(58) = y(28);
y(63) = y(28);
y(93) = y(28);
y(41) = y(37);
y(80) = y(37);
y(84) = y(37);
y(40) = y(38);
y(48) = y(38);
y(52) = y(38);
y(81) = y(38);
y(83) = y(38);
y(69) = y(38);
y(73) = y(38);
y(59) = y(39);
y(62) = y(39);
y(82) = y(39);
y(51) = y(49);
y(70) = y(49);
y(72) = y(49);
y(60) = y(50);
y(61) = y(50);
y(71) = y(50);
w=1./sqrt(1+k*y.^2);

%% ======================== subfunction "randchoice" ======================
function randchoice = randchoice(set, n)   %% Randomly choose n elements of a set
Nsupport = max(size(set));
qwe = randperm(Nsupport);
mypermute = set(qwe);
randchoice = mypermute(1:n);

%% ======================== subfunction "moments" ======================
function moments = moments(design, n)   %% Compute the first few moments
for q=1:3
    moments(q) = sum((design(:,1).^(2*q)).*(design(:,2)))/n;
end

%% ======================== subfunction "rep" ======================
%% Repeats the elents of X with frequencies in N
function rep = rep(X,Nsupport)
rep = [];
for i=1:length(Nsupport)
    rep=[rep; srep(X(i),Nsupport(i))];
end
 
%% ======================== subfunction "srep" ======================
%% Called by rep; does the same thing as rep if X and N have just one element each
function srep = srep(x,n) 
srep = x*ones(n,1);

%% ================== subfunction "Fint" ======================
function int = Fint(a,b, df) 
%% Computes the integral over [0, a] of (z^b)*density(z) where the
%% density is Cauchy ('t1') or Normal ('Normal')
%% Those for t1 are gotten from: syms x; a=int(x^b/(1+x^2),x)
if strcmp(df,'t1')
    if (b == 2)
        out = @(x) (x - atan(x));
    elseif (b == 4)
       out = @(x) (atan(x) - x + x.^3/3); 
    elseif (b == 6)
       out = @(x) (x - atan(x) - x.^3/3 + x.^5/5); 
	elseif (b == 8)
       out = @(x) (atan(x) - x + x.^3/3 - x.^5/5 + x.^7/7); 
	elseif (b == 10)
       out = @(x) (x - atan(x) - x.^3/3 + x.^5/5 - x.^7/7 + x.^9/9); 
    end
    int = out(a)/pi;
elseif strcmp(df,'t3')
    if (b == 2)
        out = @(x) ((3*3^(1/2)*atan((3^(1/2)*x)/3))/2 - (9*x)./(2*(x.^2 + 3)));
    elseif (b == 4)
       out = @(x) (9*x - (27*3^(1/2)*atan((3^(1/2)*x)/3))/2 + (27*x)./(2*(x.^2 + 3))); 
    elseif (b == 6)
       out = @(x) ((135*3^(1/2)*atan((3^(1/2)*x)/3))/2 - 54*x - (81*x)./(2*(x.^2 + 3)) + 3*x.^3); 
	elseif (b == 8)
       out = @(x) (243*x - (567*3^(1/2)*atan((3^(1/2)*x)/3))/2 + (243*x)./(2*(x.^2 + 3)) - 18*x.^3 + (9*x.^5)/5); 
	elseif (b == 10)
       out = @(x) ((2187*3^(1/2)*atan((3^(1/2)*x)/3))/2 - 972*x - (729*x)./(2*(x.^2 + 3)) + 81*x.^3 - (54*x.^5)/5 + (9*x.^7)/7); 
    end
    int = 2*out(a)/(pi*sqrt(3));    
elseif strcmp(df, 'Normal') 
    out = @(x)(2^(b/2-1))*gamma((b+1)/2)*gamcdf((x.^2)/2, (b+1)/2)/sqrt(pi);
    int = out(a);
elseif strcmp(df, 'N9') 
    out = @(x)(2^(b/2-1))*gamma((b+1)/2)*gamcdf((x.^2)/2, (b+1)/2)/sqrt(pi);
    int = (3^b)*out(a/3);
end

%% ================== subfunction "B1" ======================
function [outPhi, outdf, outF] = B1(c, df, eps)
outPhi = Fint(c, 2, 'Normal')./(c.^2) - 3*Fint(c, 4, 'Normal')./(c.^4) + ...
    5*Fint(c,6, 'Normal')./(3*(c.^6));
outdf = Fint(c, 2, df)./(c.^2) - 3*Fint(c, 4, df)./(c.^4) + ...
    5*Fint(c,6, df)./(3*(c.^6));
outF = (1 - eps)*outPhi + eps*outdf;

%% ================== subfunction "B2" ======================
function [outPhi, outdf, outF] = B2(c, df, eps)
outPhi = 6*Fint(c, 2, 'Normal')./(c.^2) - 5*Fint(c, 4, 'Normal')./(c.^4);
outdf = 6*Fint(c, 2, df)./(c.^2) - 5*Fint(c, 4, df)./(c.^4);
outF = (1 - eps)*outPhi + eps*outdf;

%% ================== subfunction "B3" ======================
function [outPhi, outdf, outF] = B3(c, df, eps)
outPhi = Fint(c, 2, 'Normal')./(c.^2) - 4*Fint(c, 4, 'Normal')./(c.^4) + ...
    6*Fint(c,6, 'Normal')./(c.^6) - 4*Fint(c,8, 'Normal')./(c.^8) + ...
    Fint(c,10, 'Normal')./(c.^10);
outdf = Fint(c, 2, df)./(c.^2) - 4*Fint(c, 4, df)./(c.^4) + ...
    6*Fint(c,6, df)./(c.^6) - 4*Fint(c,8, df)./(c.^8) + ...
    Fint(c,10, df)./(c.^10);
outF = (1 - eps)*outPhi + eps*outdf;

%% ================== subfunction "H" ======================
function [outPhi, outdf, outF] = H(c, df, eps)
outPhi = Fint(c, 2, 'Normal')./(c.^2) + normcdf(-c);
if strcmp(df,'t1')
    outdf = Fint(c, 2, 't1')./(c.^2) + tcdf(-c,1);
elseif strcmp(df,'t3')
    outdf = Fint(c, 2, 't3')./(c.^2) + tcdf(-c,3);
elseif strcmp(df,'N9')
    outdf = Fint(c, 2, 'N9')./(c.^2) + normcdf(-c,0,3);
end
outF = (1 - eps)*outPhi + eps*outdf;



